#!/usr/bin/perl
# Streetwikibot V1.0

use warnings;
use strict;
use Mail::Sendmail;
use LWP::Simple;
use LWP::UserAgent;
use HTTP::Cookies;
use MediaWiki::API;

# configuration

our $BASE_DIR = "/var/mercurial/metagov/streetwiki";
our $EXPORT;

unless (-e $BASE_DIR)
{
	mkdir($BASE_DIR,0700) || die "Can't create $BASE_DIR : $!";;
}

if (!-e ".streetwikibotrc") {
    die 'No configuration file found !

Create a .streetwikibotrc file at the root of the script and add the following parameters in it : 
our $BASE_URL = "okidoke.referata.com" # the host or ip address of the streetwiki
our $BROWSER = "Streetwikibot/1.0"  # the name and version of the bot that will be kept in the server logs
our $NUM_PAGES = 500 #number of pages by batch 
our $NUM_TURNS = 10 # number of batch to run
our $NS_NUM = 10 # number of the namespace to backup';
}

require(".streetwikibotrc");

our (
    $BASE_URL,$BROWSER,$NUM_PAGES,$NUM_TURNS,$NS_NUM
); 

# end of configuration

open (LOGS, ">>$BASE_DIR/$BASE_URL.log");
select((select(LOGS), $|=1)[0]);
	
my $mw = MediaWiki::API->new();
#$mw->{config}->{api_url} = 'http://okidoke.referata.com/w/api.php';
$mw->{config}->{api_url} = "http://$BASE_URL/w/api.php";

get_pages($mw);
my $xml_content;
$xml_content = send_request("w/index.php?title=Special:Export&pages=$EXPORT&action=submit&history","saving ".$BASE_URL." to $BASE_URL.xml",1);
if ($xml_content)
{
    open(my $xml_file,'>',"$BASE_DIR/$BASE_URL.xml") || die "Can't open : $BASE_DIR/$BASE_URL.xml $!";
    print $xml_file $xml_content;
    close($xml_file);
} else {
    exit 0;   
}

sub get_pages {
    my ($mw) = @_;
    $mw->list ( { action => 'query',
              list => 'allpages',
              apnamespace => $NS_NUM,
              aplimit=> $NUM_PAGES },
            { max => $NUM_TURNS, hook => \&loop_pages } )
    || die log_it($mw->{error}->{code} . ': ' . $mw->{error}->{details});
}

sub loop_pages {
    my ($ref) = @_;
    foreach (@$ref) {
        if ($_->{title} ne '%s') 
        {
            $EXPORT.= $_->{title}.'%0A';
        } 
    }
}

sub log_it
{
	my($comment) = @_;
	my $date_log = localtime(time());
	my $log_line = "$date_log: $comment\n";
	print LOGS $log_line;
}

sub send_request
{
	my ($path,$comment,$post) = @_;
    my $ua = LWP::UserAgent->new;
    $path =~ s/ /_/g;
    $ua->agent($BROWSER . $ua->agent);
    $ua->cookie_jar(HTTP::Cookies->new(file => "$BASE_DIR/$BASE_URL.cookies", autosave => 1));
    $ua->requests_redirectable;

	my ($req,$res);
	if ($post)
	{
		$req = HTTP::Request->new(POST => "http://$BASE_URL/$path");
		$req->content_type('application/x-www-form-urlencoded');
		$req->content($post);
	}
	else
	{
		$req = HTTP::Request->new(GET => "http://$BASE_URL/$path");
	}
	$res = $ua->request($req);
	if ($res->is_success)
	{	
		log_it($comment) unless ($comment eq "");
		return $res->decoded_content;
	}
	else
	{
		log_it("Error on $comment : ".$res->status_line);
		return 0;
	}
}

close(LOGS);

exit 0;
